package com.ruoyi.system.controller;

import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.PostMapping;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.core.utils.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.GetMapping;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import java.util.List;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.domain.SysMenu;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.service.ISysMenuService;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.web.controller.BaseController;

@RestController
@RequestMapping({ "/menu" })
public class SysMenuController extends BaseController
{
    @Autowired
    private ISysMenuService menuService;
    
    @RequiresPermissions({ "system:menu:list" })
    @GetMapping({ "/list" })
    public AjaxResult list(final SysMenu menu) {
        final Long userId = SecurityUtils.getUserId();
        final List<SysMenu> menus = this.menuService.selectMenuList(menu, userId);
        return this.success((Object)menus);
    }
    
    @RequiresPermissions({ "system:menu:query" })
    @GetMapping({ "/{menuId}" })
    public AjaxResult getInfo(@PathVariable final Long menuId) {
        return this.success((Object)this.menuService.selectMenuById(menuId));
    }
    
    @GetMapping({ "/treeselect" })
    public AjaxResult treeselect(final SysMenu menu) {
        final Long userId = SecurityUtils.getUserId();
        final List<SysMenu> menus = this.menuService.selectMenuList(menu, userId);
        return this.success((Object)this.menuService.buildMenuTreeSelect((List)menus));
    }
    
    @GetMapping({ "/roleMenuTreeselect/{roleId}" })
    public AjaxResult roleMenuTreeselect(@PathVariable("roleId") final Long roleId) {
        final Long userId = SecurityUtils.getUserId();
        final List<SysMenu> menus = this.menuService.selectMenuList(userId);
        final AjaxResult ajax = AjaxResult.success();
        ajax.put("checkedKeys", (Object)this.menuService.selectMenuListByRoleId(roleId));
        ajax.put("menus", (Object)this.menuService.buildMenuTreeSelect((List)menus));
        return ajax;
    }
    
    @RequiresPermissions({ "system:menu:add" })
    @Log(title = "菜单管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody final SysMenu menu) {
        if (!this.menuService.checkMenuNameUnique(menu)) {
            return this.error("新增菜单'" + menu.getMenuName() + "'失败，菜单名称已存在");
        }
        if ("0".equals(menu.getIsFrame()) && !StringUtils.ishttp(menu.getPath())) {
            return this.error("新增菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        menu.setCreateBy(SecurityUtils.getUsername());
        return this.toAjax(this.menuService.insertMenu(menu));
    }
    
    @RequiresPermissions({ "system:menu:edit" })
    @Log(title = "菜单管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody final SysMenu menu) {
        if (!this.menuService.checkMenuNameUnique(menu)) {
            return this.error("修改菜单'" + menu.getMenuName() + "'失败，菜单名称已存在");
        }
        if ("0".equals(menu.getIsFrame()) && !StringUtils.ishttp(menu.getPath())) {
            return this.error("修改菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        }
        if (menu.getMenuId().equals(menu.getParentId())) {
            return this.error("修改菜单'" + menu.getMenuName() + "'失败，上级菜单不能选择自己");
        }
        menu.setUpdateBy(SecurityUtils.getUsername());
        return this.toAjax(this.menuService.updateMenu(menu));
    }
    
    @RequiresPermissions({ "system:menu:remove" })
    @Log(title = "菜单管理", businessType = BusinessType.DELETE)
    @DeleteMapping({ "/{menuId}" })
    public AjaxResult remove(@PathVariable("menuId") final Long menuId) {
        if (this.menuService.hasChildByMenuId(menuId)) {
            return this.warn("存在子菜单,不允许删除");
        }
        if (this.menuService.checkMenuExistRole(menuId)) {
            return this.warn("菜单已分配,不允许删除");
        }
        return this.toAjax(this.menuService.deleteMenuById(menuId));
    }
    
    @GetMapping({ "getRouters" })
    public AjaxResult getRouters() {
        final Long userId = SecurityUtils.getUserId();
        final List<SysMenu> menus = this.menuService.selectMenuTreeByUserId(userId);
        return this.success((Object)this.menuService.buildMenus((List)menus));
    }
}
